@@ -8,7 +8,7 @@ Config __init__ file
 
 from .api_config import RecreationBookingConfig, RIDBConfig, STANDARD_HEADERS, USER_AGENTS
 from .cli_config import CommandLineConfig
-from .data_columns import DataColumns
+from .data_columns import CampsiteContainerFields, DataColumns
 from .file_config import FileConfig
 from .notification_config import EmailConfig, PushbulletConfig, PushoverConfig
 from .search_config import SearchConfig
