@@ -7,7 +7,7 @@ Recreation.gov Web Searching Utilities
 """
 
 from base64 import b64decode
-from datetime import datetime
+from datetime import datetime, timedelta
 from json import loads
 import logging
 from random import choice
@@ -256,7 +256,7 @@ class RecreationDotGov(BaseProvider):
         """
         api_endpoint = self._ridb_get_endpoint(path=path)
         response = requests.get(url=api_endpoint, headers=self._ridb_api_headers,
-                                params=params)
+                                params=params, timeout=30)
         try:
             assert response.status_code == 200
         except AssertionError:
@@ -482,7 +482,8 @@ class RecreationDotGov(BaseProvider):
             headers.update(choice(USER_AGENTS))
             headers.update(RecreationBookingConfig.API_REFERRERS)
             response = requests.get(url=api_endpoint, headers=headers,
-                                    params=dict(start_date=formatted_month))
+                                    params=dict(start_date=formatted_month),
+                                    timeout=30)
             assert response.status_code == 200
         except AssertionError:
             response_error = response.text
@@ -571,6 +572,8 @@ class RecreationDotGov(BaseProvider):
                     available_campsite = AvailableCampsite(
                         campsite_id=campsite_id,
                         booking_date=matching_date,
+                        booking_end_date=matching_date + timedelta(days=1),
+                        booking_nights=1,
                         campsite_site_name=campsite_name,
                         campsite_loop_name=campsite_loop,
                         campsite_type=campsite_type,
